/*
 * Copyright (C) 2005-2006 Alex Murray <pragmatine@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif /* HAVE_CONFIG_H */

#include "sensors-applet.h"
#include "sensors-applet-gconf.h"

#define DEFAULT_TIMEOUT 2000
#define DEFAULT_GRAPH_SIZE 42

/* always list current version */
static const gchar * const compatible_versions[] = {
        PACKAGE_VERSION,
        "1.7.3",
        "1.7.2"
};

#define NUM_COMPATIBLE_VERSIONS G_N_ELEMENTS(compatible_versions)

typedef enum {
        SENSORS_APPLET_GCONF_ERROR = 0,
        SENSORS_APPLET_VERSION_ERROR,
} SensorsAppletGConfError;

static const gchar * const error_titles[] = {
        N_("An error occurred loading the stored sensors data"),
        N_("Incompatible sensors configuration found")
};

static const gchar * const error_messages[] = {
        N_("An error has occurred when loading the stored sensors data. "
           "The default values will be used to recover from this error."),
        
        N_("Unfortunately the previous configuration for GNOME Sensors Applet "
          "is not compatible with this version. The existing sensors data "
          "will be overwritten with the default values for this new version.")
};

/* function to be called if an error occurs
   when loading values from gconf */
static void sensors_applet_gconf_error_occurred(SensorsAppletGConfError error) {
	GtkWidget *dialog;
        gchar *markup;

        g_debug("Error occurred: %s", error_titles[error]);
        markup = g_markup_printf_escaped("<span size=\"large\" weight=\"bold\">%s</span>\n\n%s", _(error_titles[error]), _(error_messages[error]));

	dialog = gtk_message_dialog_new_with_markup(NULL, /* no parent window */
                                                    GTK_DIALOG_MODAL,
                                                    GTK_MESSAGE_WARNING,
                                                    GTK_BUTTONS_OK,
                                                    markup);

        g_free(markup);

         /* runs dialog as modal and doesn't return until user clicks
         * button */
        gtk_dialog_run(GTK_DIALOG(dialog));
        gtk_widget_destroy(GTK_WIDGET(dialog));
}


static void sensors_applet_gconf_set_defaults(SensorsApplet *sensors_applet) {
	panel_applet_gconf_set_int(sensors_applet->applet, DISPLAY_MODE, DISPLAY_ICON_WITH_VALUE, NULL);
	panel_applet_gconf_set_int(sensors_applet->applet, LAYOUT_MODE, VALUE_BESIDE_LABEL, NULL);
	panel_applet_gconf_set_int(sensors_applet->applet, TEMPERATURE_SCALE, CELSIUS, NULL);
	panel_applet_gconf_set_int(sensors_applet->applet, TIMEOUT, DEFAULT_TIMEOUT, NULL);
	panel_applet_gconf_set_int(sensors_applet->applet, GRAPH_SIZE, DEFAULT_GRAPH_SIZE, NULL);
#ifdef HAVE_LIBNOTIFY
	panel_applet_gconf_set_bool(sensors_applet->applet, DISPLAY_NOTIFICATIONS, TRUE, NULL);
#endif        
	panel_applet_gconf_set_bool(sensors_applet->applet, IS_SETUP, FALSE, NULL);

}

/**
 * Returns TRUE is old_version is one of the compatible versions
 */
static gboolean sensors_applet_gconf_is_compatible(const gchar *old_version) {
        gint i;
        for (i = 0; i < NUM_COMPATIBLE_VERSIONS; i++) {
                if (g_ascii_strcasecmp(old_version, compatible_versions[i]) == 0) {
                        return TRUE;
                }
        }
        return FALSE;
}

void sensors_applet_gconf_setup(SensorsApplet *sensors_applet) {
        gboolean setup = FALSE;
        gchar *old_version;
        GError *error = NULL;

        setup = panel_applet_gconf_get_bool(sensors_applet->applet, 
                                            IS_SETUP, &error);
        if (error) {
                g_debug("Previous configuration not found: %s, setting up manually", error->message);
                g_error_free(error);
                error = NULL;
                setup = FALSE;
        }

	if (setup) {
                old_version = panel_applet_gconf_get_string(sensors_applet->applet,
                                                            SENSORS_APPLET_VERSION,
                                                            &error);
                /* if versions don't match or there is no saved
                 * version string then need to overwrite old config */
                if (error || NULL == old_version || 
                    (!sensors_applet_gconf_is_compatible(old_version))) {
                        if (error) {
                                g_debug("Error getting old version string: %s", error->message);
                                g_error_free(error);
                                error = NULL;
                        }
                        
                        sensors_applet_gconf_error_occurred(SENSORS_APPLET_VERSION_ERROR);
                        
                } else {
                        /* previously setup and versions match so use
                         * old values */
                        g_debug("Trying to set up sensors from gconf data");
                        if (sensors_applet_gconf_setup_sensors(sensors_applet)) {
                                g_debug("done setting up from gconf");
                                return;
                        } 
                }
        }
        /* use defaults */
        g_debug("Setting gconf defaults only");
        sensors_applet_gconf_set_defaults(sensors_applet);
}

/* gets called if are already setup so we don't have to manually go
   through and find sensors etc again */
gboolean sensors_applet_gconf_setup_sensors(SensorsApplet *sensors_applet) {
	/* everything gets stored except alarm timeout indexes, which
	   we set to -1, and visible which we set to false for all
	   parent nodes and true for all child nodes */

	GSList *paths = NULL, *ids = NULL, *labels = NULL, *interfaces = NULL, *enables = NULL, *low_values = NULL, *high_values = NULL, *alarm_enables = NULL, *alarm_commands = NULL, *alarm_timeouts = NULL, *sensor_types = NULL, *multipliers = NULL, *offsets = NULL, *icon_types = NULL, *graph_colors = NULL;
	GSList *current_path, *current_id, *current_label, *current_interface, *current_enable, *current_low_value, *current_high_value, *current_alarm_command, *current_alarm_timeout, *current_alarm_enable, *current_sensor_type, *current_multiplier, *current_offset, *current_icon_type, *current_graph_color;

	GError *error = NULL;

	paths = panel_applet_gconf_get_list(sensors_applet->applet, PATHS, 
                                            GCONF_VALUE_STRING, &error);
	if (error) {
		sensors_applet_gconf_error_occurred(SENSORS_APPLET_GCONF_ERROR);
		g_error_free(error);
		return FALSE;
	}

	ids = panel_applet_gconf_get_list(sensors_applet->applet, IDS, 
                                          GCONF_VALUE_STRING, &error);
	if (error) {
		sensors_applet_gconf_error_occurred(SENSORS_APPLET_GCONF_ERROR);
		g_error_free(error);
		return FALSE;
	}

	labels = panel_applet_gconf_get_list(sensors_applet->applet, LABELS, 
                                             GCONF_VALUE_STRING, &error);
	if (error) {	
		sensors_applet_gconf_error_occurred(SENSORS_APPLET_GCONF_ERROR);
		g_error_free(error);
		return FALSE;
	}

	interfaces = panel_applet_gconf_get_list(sensors_applet->applet, 
                                                 INTERFACES, 
                                                 GCONF_VALUE_INT, &error);
	if (error) {
		sensors_applet_gconf_error_occurred(SENSORS_APPLET_GCONF_ERROR);
		g_error_free(error);
		return FALSE;
	}

	sensor_types = panel_applet_gconf_get_list(sensors_applet->applet, 
                                                   SENSOR_TYPES, 
                                                   GCONF_VALUE_INT, &error);
	if (error) {
		sensors_applet_gconf_error_occurred(SENSORS_APPLET_GCONF_ERROR);
		g_error_free(error);
		return FALSE;
	}

	enables = panel_applet_gconf_get_list(sensors_applet->applet, ENABLES,
                                              GCONF_VALUE_BOOL, &error);
	if (error) {
		sensors_applet_gconf_error_occurred(SENSORS_APPLET_GCONF_ERROR);
		g_error_free(error);
		return FALSE;
	}

	low_values = panel_applet_gconf_get_list(sensors_applet->applet, 
                                                 LOW_VALUES, GCONF_VALUE_INT, 
                                                 &error);
	if (error) {
		sensors_applet_gconf_error_occurred(SENSORS_APPLET_GCONF_ERROR);
		g_error_free(error);
		return FALSE;
	}

	high_values = panel_applet_gconf_get_list(sensors_applet->applet, 
                                                  HIGH_VALUES, 
                                                  GCONF_VALUE_INT, &error);
	if (error) {
		sensors_applet_gconf_error_occurred(SENSORS_APPLET_GCONF_ERROR);
		g_error_free(error);
		return FALSE;
	}

	alarm_enables = panel_applet_gconf_get_list(sensors_applet->applet, 
                                                    ALARM_ENABLES, 
                                                    GCONF_VALUE_BOOL, &error);
	if (error) {
		sensors_applet_gconf_error_occurred(SENSORS_APPLET_GCONF_ERROR);
		g_error_free(error);
		return FALSE;
	}

	alarm_commands = panel_applet_gconf_get_list(sensors_applet->applet, 
                                                     ALARM_COMMANDS, 
                                                     GCONF_VALUE_STRING, 
                                                     &error);
	if (error) {
		sensors_applet_gconf_error_occurred(SENSORS_APPLET_GCONF_ERROR);
		g_error_free(error);
		return FALSE;
	}

	alarm_timeouts = panel_applet_gconf_get_list(sensors_applet->applet, 
                                                     ALARM_TIMEOUTS, 
                                                     GCONF_VALUE_INT, &error);
	if (error) {
		sensors_applet_gconf_error_occurred(SENSORS_APPLET_GCONF_ERROR);
		g_error_free(error);
		return FALSE;
	}

	multipliers = panel_applet_gconf_get_list(sensors_applet->applet, 
                                                  MULTIPLIERS, 
                                                  GCONF_VALUE_INT, &error);
	if (error) {
		sensors_applet_gconf_error_occurred(SENSORS_APPLET_GCONF_ERROR);
		g_error_free(error);
		return FALSE;
	}

	offsets = panel_applet_gconf_get_list(sensors_applet->applet, OFFSETS, 
                                              GCONF_VALUE_INT, &error);
	if (error) {
		sensors_applet_gconf_error_occurred(SENSORS_APPLET_GCONF_ERROR);
		g_error_free(error);
		return FALSE;
	}

	icon_types = panel_applet_gconf_get_list(sensors_applet->applet, 
                                                 ICON_TYPES, GCONF_VALUE_INT, 
                                                 &error);
	if (error) {
		sensors_applet_gconf_error_occurred(SENSORS_APPLET_GCONF_ERROR);
		g_error_free(error);
		return FALSE;
	}

	graph_colors = panel_applet_gconf_get_list(sensors_applet->applet, 
                                                   GRAPH_COLORS, 
                                                   GCONF_VALUE_STRING, &error);
	if (error) {
		sensors_applet_gconf_error_occurred(SENSORS_APPLET_GCONF_ERROR);
		g_error_free(error);
		return FALSE;
	}

	for (current_path = paths, current_id = ids, current_label = labels, current_interface = interfaces, current_enable = enables, current_low_value = low_values, current_high_value = high_values, current_alarm_enable = alarm_enables, current_alarm_command = alarm_commands, current_alarm_timeout = alarm_timeouts, current_sensor_type = sensor_types, current_multiplier = multipliers, current_offset = offsets, current_icon_type = icon_types, current_graph_color = graph_colors; 
	     current_path != NULL && current_id != NULL && current_label != NULL && current_enable != NULL && current_interface != NULL && current_low_value != NULL && current_high_value != NULL && current_alarm_enable != NULL && current_alarm_command != NULL && current_alarm_timeout != NULL && current_sensor_type != NULL && current_multiplier != NULL && current_offset != NULL && current_icon_type != NULL && current_graph_color != NULL;
	     current_path = g_slist_next(current_path), current_id = g_slist_next(current_id), current_label = g_slist_next(current_label), current_enable = g_slist_next(current_enable), current_interface = g_slist_next(current_interface), current_low_value = g_slist_next(current_low_value), current_high_value = g_slist_next(current_high_value), current_alarm_enable = g_slist_next(current_alarm_enable), current_alarm_command = g_slist_next(current_alarm_command), current_alarm_timeout = g_slist_next(current_alarm_timeout), current_sensor_type = g_slist_next(current_sensor_type), current_multiplier = g_slist_next(current_multiplier), current_offset = g_slist_next(current_offset), current_icon_type = g_slist_next(current_icon_type), current_graph_color = g_slist_next(current_graph_color)) {

		g_debug("trying to add sensor from gconf data: %s\n", (gchar *)(current_id->data));
		sensors_applet_add_sensor_full_details(sensors_applet,
                                                       (gchar *)(current_path->data), 
						       (gchar *)(current_id->data), 
						       (gchar *)(current_label->data), 
						       GPOINTER_TO_UINT(current_interface->data),
						       GPOINTER_TO_UINT(current_sensor_type->data),
						       GPOINTER_TO_INT(current_enable->data),
						       (gdouble)(GPOINTER_TO_INT(current_low_value->data) / 1000.0),
						       (gdouble)(GPOINTER_TO_INT(current_high_value->data) / 1000.0),
						       GPOINTER_TO_INT(current_alarm_enable->data),
						       (gchar *)(current_alarm_command->data),
						       GPOINTER_TO_INT(current_alarm_timeout->data),
						       (gdouble)(GPOINTER_TO_INT(current_multiplier->data) / 1000.0),
						       (gdouble)(GPOINTER_TO_INT(current_offset->data) / 1000.0),
						       (SensorType)GPOINTER_TO_UINT(current_icon_type->data),
                                                       (gchar *)(current_graph_color->data)
                                                       
                        );

	}
        
       	g_slist_foreach(paths, (GFunc)g_free, NULL);
	g_slist_free(paths);
       	g_slist_foreach(ids, (GFunc)g_free, NULL);
	g_slist_free(ids);
       	g_slist_foreach(labels, (GFunc)g_free, NULL);
	g_slist_free(labels);
	g_slist_free(interfaces);
	g_slist_free(sensor_types);
	g_slist_free(enables);
	g_slist_free(low_values);
	g_slist_free(high_values);
	g_slist_free(alarm_enables);
       	g_slist_foreach(alarm_commands, (GFunc)g_free, NULL);
	g_slist_free(alarm_commands);
	g_slist_free(alarm_timeouts);
	g_slist_free(multipliers);
	g_slist_free(offsets);
	g_slist_free(icon_types);
       	g_slist_foreach(graph_colors, (GFunc)g_free, NULL);
	g_slist_free(graph_colors);

	return TRUE;
}


gboolean sensors_applet_gconf_save_sensors(SensorsApplet *sensors_applet) {
	/* write everything to gconf except VISIBLE and
	   ALARM_TIMEOUT_INDEX */
	/* for stepping through GtkTreeStore data structure */
	GtkTreeIter interfaces_iter, sensors_iter;
	gboolean not_end_of_interfaces = TRUE, not_end_of_sensors = TRUE;

	GSList *paths = NULL, *ids = NULL, *labels = NULL, *interfaces = NULL, *enables = NULL, *low_values = NULL, *high_values = NULL, *alarm_enables = NULL, *alarm_commands = NULL, *alarm_timeouts = NULL, *sensor_types = NULL, *multipliers = NULL, *offsets = NULL, *icon_types = NULL, * graph_colors = NULL;
	gchar *current_path, *current_id, *current_label, *current_alarm_command, *current_graph_color;
	gboolean current_enable, current_alarm_enable;
	gdouble current_low_value, current_high_value, current_multiplier, current_offset;
	guint current_interface, current_alarm_timeout, current_sensor_type, current_icon_type;
	
	GError *error = NULL;
	/* now step through the GtkTreeStore sensors to
	   find which sensors are enabled */
	for (gtk_tree_model_get_iter_first(GTK_TREE_MODEL(sensors_applet->sensors), &interfaces_iter); not_end_of_interfaces; not_end_of_interfaces = gtk_tree_model_iter_next(GTK_TREE_MODEL(sensors_applet->sensors), &interfaces_iter)) {
		// store a gconf key for this interface
		gtk_tree_model_get(GTK_TREE_MODEL(sensors_applet->sensors), 
				   &interfaces_iter,
				   ID_COLUMN, &current_id,
				   -1);

		panel_applet_gconf_set_bool(sensors_applet->applet, current_id, TRUE, NULL);
		g_free(current_id);
		
		/* reset sensors sentinel */
		not_end_of_sensors = TRUE;
		
		for (gtk_tree_model_iter_children(GTK_TREE_MODEL(sensors_applet->sensors), &sensors_iter, &interfaces_iter); not_end_of_sensors; not_end_of_sensors = gtk_tree_model_iter_next(GTK_TREE_MODEL(sensors_applet->sensors), &sensors_iter)) {
			gtk_tree_model_get(GTK_TREE_MODEL(sensors_applet->sensors), 
					   &sensors_iter,
					   PATH_COLUMN, &current_path,
					   ID_COLUMN, &current_id,
					   LABEL_COLUMN, &current_label,
					   INTERFACE_COLUMN, &current_interface,
					   SENSOR_TYPE_COLUMN, &current_sensor_type,
					   ENABLE_COLUMN, &current_enable,
					   LOW_VALUE_COLUMN, &current_low_value,
					   HIGH_VALUE_COLUMN, &current_high_value,
					   ALARM_ENABLE_COLUMN, &current_alarm_enable,
					   ALARM_COMMAND_COLUMN, &current_alarm_command,
					   ALARM_TIMEOUT_COLUMN, &current_alarm_timeout,
					   MULTIPLIER_COLUMN, &current_multiplier,
					   OFFSET_COLUMN, &current_offset,
					   ICON_TYPE_COLUMN, &current_icon_type,
                                           GRAPH_COLOR_COLUMN, &current_graph_color,
					   -1);

			/* prepend values as this is faster then just
			   reverse list when finished */
			paths = g_slist_prepend(paths, current_path);
			ids = g_slist_prepend(ids, current_id);
			labels = g_slist_prepend(labels, current_label);
			interfaces = g_slist_prepend(interfaces, GUINT_TO_POINTER(current_interface));
			enables = g_slist_prepend(enables, GINT_TO_POINTER(current_enable));
			low_values = g_slist_prepend(low_values, GINT_TO_POINTER((gint)(current_low_value * 1000)));
			high_values = g_slist_prepend(high_values, GINT_TO_POINTER((gint)(current_high_value * 1000)));
			alarm_enables = g_slist_prepend(alarm_enables, GINT_TO_POINTER(current_alarm_enable));
			alarm_commands = g_slist_prepend(alarm_commands, current_alarm_command);
			alarm_timeouts = g_slist_prepend(alarm_timeouts, GINT_TO_POINTER(current_alarm_timeout));
			sensor_types = g_slist_prepend(sensor_types, GUINT_TO_POINTER(current_sensor_type));
			multipliers = g_slist_prepend(multipliers, GINT_TO_POINTER((gint)(current_multiplier * 1000)));
			offsets = g_slist_prepend(offsets, GINT_TO_POINTER((gint)(current_offset * 1000)));
			icon_types = g_slist_prepend(icon_types, GUINT_TO_POINTER(current_icon_type));
			graph_colors = g_slist_prepend(graph_colors, current_graph_color);
		}
	}

	/* keep lists in original order */
	paths = g_slist_reverse(paths);
	ids = g_slist_reverse(ids);
	labels = g_slist_reverse(labels);
	interfaces = g_slist_reverse(interfaces);
	enables = g_slist_reverse(enables);
	low_values = g_slist_reverse(low_values);
	high_values = g_slist_reverse(high_values);
	alarm_enables = g_slist_reverse(alarm_enables);
	alarm_commands = g_slist_reverse(alarm_commands);
	alarm_timeouts = g_slist_reverse(alarm_timeouts);
	sensor_types = g_slist_reverse(sensor_types);
	multipliers = g_slist_reverse(multipliers);
	offsets = g_slist_reverse(offsets);
	icon_types = g_slist_reverse(icon_types);
	graph_colors = g_slist_reverse(graph_colors);

	panel_applet_gconf_set_list(sensors_applet->applet, PATHS, GCONF_VALUE_STRING, paths, &error);
	if (error) {
		g_error_free(error);
		return FALSE;
	}

	panel_applet_gconf_set_list(sensors_applet->applet, IDS, GCONF_VALUE_STRING, ids, &error);
	if (error) {
		g_error_free(error);
		return FALSE;
	}

	panel_applet_gconf_set_list(sensors_applet->applet, LABELS, GCONF_VALUE_STRING, labels, &error);
	if (error) {
		g_error_free(error);
		return FALSE;
	}

	panel_applet_gconf_set_list(sensors_applet->applet, INTERFACES, GCONF_VALUE_INT, interfaces, &error);
	if (error) {
		g_error_free(error);
		return FALSE;
	}
	
	panel_applet_gconf_set_list(sensors_applet->applet, SENSOR_TYPES, GCONF_VALUE_INT, sensor_types, &error);
	if (error) {
		g_error_free(error);
		return FALSE;
	}
	
	panel_applet_gconf_set_list(sensors_applet->applet, ENABLES, GCONF_VALUE_BOOL, enables, &error);
	if (error) {
		g_error_free(error);
		return FALSE;
	}
	
	panel_applet_gconf_set_list(sensors_applet->applet, LOW_VALUES, GCONF_VALUE_INT, low_values, &error);
	if (error) {
		g_error_free(error);
		return FALSE;
	}
	
	panel_applet_gconf_set_list(sensors_applet->applet, HIGH_VALUES, GCONF_VALUE_INT, high_values, &error);
	if (error) {
		g_error_free(error);
		return FALSE;
	}
	
	panel_applet_gconf_set_list(sensors_applet->applet, ALARM_ENABLES, GCONF_VALUE_BOOL, alarm_enables, &error);
	if (error) {
		g_error_free(error);
		return FALSE;
	}
	
	panel_applet_gconf_set_list(sensors_applet->applet, ALARM_COMMANDS, GCONF_VALUE_STRING, alarm_commands, &error);
	if (error) {
		g_error_free(error);
		return FALSE;
	}
	
	panel_applet_gconf_set_list(sensors_applet->applet, ALARM_TIMEOUTS, GCONF_VALUE_INT, alarm_timeouts, &error);
	if (error) {
		g_error_free(error);
		return FALSE;
	}
	
	panel_applet_gconf_set_list(sensors_applet->applet, MULTIPLIERS, GCONF_VALUE_INT, multipliers, &error);
	if (error) {
		g_error_free(error);
		return FALSE;
	}
	
	panel_applet_gconf_set_list(sensors_applet->applet, OFFSETS, GCONF_VALUE_INT, offsets, &error);
	if (error) {
		g_error_free(error);
		return FALSE;
	}
	
	panel_applet_gconf_set_list(sensors_applet->applet, ICON_TYPES, GCONF_VALUE_INT, icon_types, &error);
	if (error) {
		g_error_free(error);
		return FALSE;
	}

	panel_applet_gconf_set_list(sensors_applet->applet, GRAPH_COLORS, GCONF_VALUE_STRING, graph_colors, &error);
	if (error) {
		g_error_free(error);
		return FALSE;
	}

	panel_applet_gconf_set_string(sensors_applet->applet,
                                      SENSORS_APPLET_VERSION,
                                      PACKAGE_VERSION, &error);
        if (error) {
                g_error_free(error);
                return FALSE;
        }

	/* need to free allocated strings */
	g_slist_foreach(paths, (GFunc)g_free, NULL);
	g_slist_free(paths);
	g_slist_foreach(ids, (GFunc)g_free, NULL);
	g_slist_free(ids);
	g_slist_foreach(labels, (GFunc)g_free, NULL);
	g_slist_free(labels);
	g_slist_free(interfaces);
	g_slist_free(enables);
	g_slist_free(low_values);
	g_slist_free(high_values);
	g_slist_free(alarm_enables);
	g_slist_foreach(alarm_commands, (GFunc)g_free, NULL);
	g_slist_free(alarm_commands);
	g_slist_free(alarm_timeouts);
	g_slist_free(sensor_types);
	g_slist_free(multipliers);
	g_slist_free(offsets);
	g_slist_free(icon_types);
	g_slist_foreach(graph_colors, (GFunc)g_free, NULL);
	g_slist_free(graph_colors);

	return TRUE;
}

